@@ -11,8 +11,6 @@
 # 
 # Example to find the equivalent information using OWSLib:
 # 
-from __future__ import absolute_import
-from __future__ import print_function
 from owslib.wcs import WebCoverageService
 wcs=WebCoverageService('http://cida.usgs.gov/thredds/wcs/prism',version='1.0.0')
 # Take a look at the contents (coverages) of the wcs.
