@@ -53,18 +53,19 @@ class WebCoverageService_2_0_0(WCSBase):
         else:
             raise KeyError("No content named %s" % name)
 
-    def __init__(self, url, xml, cookies, auth=None):
-        super(WebCoverageService_2_0_0, self).__init__(auth=auth)
+    def __init__(self, url, xml, cookies, auth=None, timeout=30, headers=None):
+        super(WebCoverageService_2_0_0, self).__init__(auth=auth, headers=headers)
         self.version = "2.0.0"
         self.url = url
         self.cookies = cookies
+        self.timeout = timeout
         self.ows_common = OwsCommon(version="2.0.0")
         # initialize from saved capability document or access the server
-        reader = WCSCapabilitiesReader(self.version, self.cookies, self.auth)
+        reader = WCSCapabilitiesReader(self.version, self.cookies, self.auth, headers=self.headers)
         if xml:
             self._capabilities = reader.readString(xml)
         else:
-            self._capabilities = reader.read(self.url)
+            self._capabilities = reader.read(self.url, self.timeout)
 
         # check for exceptions
         se = self._capabilities.find("ServiceException")
@@ -113,14 +114,6 @@ class WebCoverageService_2_0_0(WCSBase):
             items.append((item, self.contents[item]))
         return items
 
-    def __makeString(self, value):
-        # using repr unconditionally breaks things in some circumstances if a value is already a string
-        if type(value) is not str:
-            sval = repr(value)
-        else:
-            sval = value
-        return sval
-
     def getCoverage(
         self,
         identifier=None,
@@ -138,6 +131,7 @@ class WebCoverageService_2_0_0(WCSBase):
         resz=None,
         parameter=None,
         method="Get",
+        timeout=30,
         **kwargs
     ):
         """Request and return a coverage from the WCS as a file-like object
@@ -221,19 +215,9 @@ class WebCoverageService_2_0_0(WCSBase):
             data += param_list_to_url_string(sizes, 'size')
         log.debug("WCS 2.0.0 DEBUG: Second part of URL: %s" % data)
 
-        u = openURL(base_url, data, method, self.cookies, auth=self.auth)
+        u = openURL(base_url, data, method, self.cookies, auth=self.auth, timeout=timeout, headers=self.headers)
         return u
 
-    def is_number(self, s):
-        """simple helper to test if value is number as requests with numbers dont
-        need quote marks
-        """
-        try:
-            float(s)
-            return True
-        except ValueError:
-            return False
-
     def getOperationByName(self, name):
         """Return a named operation item."""
         for item in self.operations:
