@@ -33,7 +33,7 @@ class ServiceException(Exception):
 class WCSBase(object):
     """Base class to be subclassed by version dependent WCS classes. Provides 'high-level'
     version independent methods"""
-    def __new__(self, url, xml, cookies, auth=None):
+    def __new__(self, url, xml, cookies, auth=None, headers=None):
         """ overridden __new__ method
 
         @type url: string
@@ -41,22 +41,25 @@ class WCSBase(object):
         @type xml: string
         @param xml: elementtree object
         @param auth: instance of owslib.util.Authentication
+        @param headers: dict for geoserver's request's headers
         @return: inititalised WCSBase object
         """
         obj = object.__new__(self)
-        obj.__init__(url, xml, cookies, auth=auth)
+        obj.__init__(url, xml, cookies, auth=auth, headers=headers)
         self.cookies = cookies
+        self.headers = headers
         self._describeCoverage = {}  # cache for DescribeCoverage responses
         return obj
 
-    def __init__(self, auth=None):
+    def __init__(self, auth=None, headers=None):
         self.auth = auth or Authentication()
+        self.headers = headers
 
     def getDescribeCoverage(self, identifier):
         ''' returns a describe coverage document - checks the internal cache to see if it has been fetched before '''
         if identifier not in list(self._describeCoverage.keys()):
             reader = DescribeCoverageReader(
-                self.version, identifier, self.cookies, self.auth)
+                self.version, identifier, self.cookies, self.auth, self.headers)
             self._describeCoverage[identifier] = reader.read(self.url)
         return self._describeCoverage[identifier]
 
@@ -65,7 +68,7 @@ class WCSCapabilitiesReader(object):
     """Read and parses WCS capabilities document into a lxml.etree infoset
     """
 
-    def __init__(self, version=None, cookies=None, auth=None):
+    def __init__(self, version=None, cookies=None, auth=None, headers=None):
         """Initialize
         @type version: string
         @param version: WCS Version parameter e.g '1.0.0'
@@ -73,6 +76,7 @@ class WCSCapabilitiesReader(object):
         self.version = version
         self._infoset = None
         self.cookies = cookies
+        self.headers = headers
         self.auth = auth or Authentication()
 
     def capabilities_url(self, service_url):
@@ -109,7 +113,7 @@ class WCSCapabilitiesReader(object):
         @return: An elementtree tree representation of the capabilities document
         """
         request = self.capabilities_url(service_url)
-        u = openURL(request, timeout=timeout, cookies=self.cookies, auth=self.auth)
+        u = openURL(request, timeout=timeout, cookies=self.cookies, auth=self.auth, headers=self.headers)
         return etree.fromstring(u.read())
 
     def readString(self, st):
@@ -124,7 +128,7 @@ class DescribeCoverageReader(object):
     """Read and parses WCS DescribeCoverage document into a lxml.etree infoset
     """
 
-    def __init__(self, version, identifier, cookies, auth=None):
+    def __init__(self, version, identifier, cookies, auth=None, headers=None):
         """Initialize
         @type version: string
         @param version: WCS Version parameter e.g '1.0.0'
@@ -133,6 +137,7 @@ class DescribeCoverageReader(object):
         self._infoset = None
         self.identifier = identifier
         self.cookies = cookies
+        self.headers = headers
         self.auth = auth or Authentication()
 
     def descCov_url(self, service_url):
@@ -186,5 +191,5 @@ class DescribeCoverageReader(object):
         """
 
         request = self.descCov_url(service_url)
-        u = openURL(request, cookies=self.cookies, timeout=timeout, auth=self.auth)
+        u = openURL(request, cookies=self.cookies, timeout=timeout, auth=self.auth, headers=self.headers)
         return etree.fromstring(u.read())
