@@ -15,7 +15,7 @@ from datetime import datetime, timedelta
 import pytz
 from owslib.etree import etree, ParseError
 from owslib.namespaces import Namespaces
-from urllib.parse import urlsplit, urlencode, urlparse, parse_qs, urlunparse, parse_qsl
+from urllib.parse import urlsplit, urlencode, urlparse, parse_qs, urlunparse, parse_qsl, unquote
 import copy
 
 from io import StringIO, BytesIO
@@ -24,6 +24,7 @@ import re
 from copy import deepcopy
 import warnings
 import requests
+from requests.auth import AuthBase
 import codecs
 
 """
@@ -170,14 +171,17 @@ def openURL(url_base, data=None, method='Get', cookies=None, username=None, pass
             auth.password = password
         if cert:
             auth.cert = cert
-        if verify and not auth.verify:
-            auth.verify = verify
+        verify = verify and auth.verify
     else:
         auth = Authentication(username, password, cert, verify)
+
     if auth.username and auth.password:
         rkwargs['auth'] = (auth.username, auth.password)
+    elif auth.auth_delegate is not None:
+        rkwargs['auth'] = auth.auth_delegate
+
     rkwargs['cert'] = auth.cert
-    rkwargs['verify'] = auth.verify
+    rkwargs['verify'] = verify
 
     # FIXUP for WFS in particular, remove xml style namespace
     # @TODO does this belong here?
@@ -193,7 +197,7 @@ def openURL(url_base, data=None, method='Get', cookies=None, username=None, pass
         rkwargs['data'] = data
 
     elif method.lower() == 'get':
-        rkwargs['params'] = data
+        rkwargs['params'] = unquote(data) if data else None
 
     else:
         raise ValueError("Unknown method ('%s'), expected 'get' or 'post'" % method)
@@ -293,7 +297,7 @@ def add_namespaces(root, ns_keys):
         # We can't re-add an existing namespaces.  Get a list of current
         # namespaces in use
         existing_namespaces = set()
-        for elem in root.getiterator():
+        for elem in root.iter():
             if elem.tag[0] == "{":
                 uri, tag = elem.tag[1:].split("}")
                 existing_namespaces.add(namespaces.get_namespace_from_url(uri))
@@ -420,6 +424,8 @@ def http_post(url=None, request=None, lang='en-US', timeout=10, username=None, p
         auth = Authentication(username, password)
     if auth.username is not None and auth.password is not None:
         rkwargs['auth'] = (auth.username, auth.password)
+    elif auth.auth_delegate is not None:
+        rkwargs['auth'] = auth.auth_delegate
     rkwargs['verify'] = auth.verify
     rkwargs['cert'] = auth.cert
 
@@ -452,6 +458,8 @@ def http_get(*args, **kwargs):
     # Build keyword args for call to requests.get()
     if auth.username and auth.password:
         rkwargs.setdefault('auth', (auth.username, auth.password))
+    elif auth.auth_delegate is not None:
+        rkwargs['auth'] = auth.auth_delegate
     else:
         rkwargs.setdefault('auth', None)
     rkwargs.setdefault('cert', rkwargs.get('cert'))
@@ -592,6 +600,10 @@ def dump(obj, prefix=''):
 def getTypedValue(data_type, value):
     '''Utility function to cast a string value to the appropriate XSD type. '''
 
+    # If the default value is empty
+    if value is None:
+        return
+
     if data_type == 'boolean':
         return True if value.lower() == 'true' else False
     elif data_type == 'integer':
@@ -786,22 +798,43 @@ def datetime_from_ansi(ansi):
     return datumOrigin + timedelta(ansi)
 
 
-def param_list_to_url_string(self, param_list, param_name):
-    """Converts list of tuples for certain WCS GetCoverage keyword arguments (subsets, resolutions, sizes) to a url-enconded
-    string
+def is_number(s):
+    """simple helper to test if value is number as requests with numbers don't
+    need quote marks
+    """
+    try:
+        float(s)
+        return True
+    except ValueError:
+        return False
+
+
+def makeString(value):
+    # using repr unconditionally breaks things in some circumstances if a
+    # value is already a string
+    if type(value) is not str:
+        sval = repr(value)
+    else:
+        sval = value
+    return sval
+
+
+def param_list_to_url_string(param_list, param_name):
+    """Converts list of tuples for certain WCS GetCoverage keyword arguments
+    (subsets, resolutions, sizes) to a url-encoded string
     """
     string = ''
     for param in param_list:
         if len(param) > 2:
-            if not self.is_number(param[1]):
-                string += "&" + urlencode({param_name: param[0] + '("' + self.__makeString(param[1]) + '","' + self.__makeString(param[2]) + '")'})  # noqa
+            if not is_number(param[1]):
+                string += "&" + urlencode({param_name: param[0] + '("' + makeString(param[1]) + '","' + makeString(param[2]) + '")'})  # noqa
             else:
-                string += "&" + urlencode({param_name: param[0] + "(" + self.__makeString(param[1]) + "," + self.__makeString(param[2]) + ")"})  # noqa
+                string += "&" + urlencode({param_name: param[0] + "(" + makeString(param[1]) + "," + makeString(param[2]) + ")"})  # noqa
         else:
-            if not self.is_number(param[1]):
-                string += "&" + urlencode({param_name: param[0] + '("' + self.__makeString(param[1]) + '")'})
+            if not is_number(param[1]):
+                string += "&" + urlencode({param_name: param[0] + '("' + makeString(param[1]) + '")'})  # noqa
             else:
-                string += "&" + urlencode({param_name: param[0] + "(" + self.__makeString(param[1]) + ")"})
+                string += "&" + urlencode({param_name: param[0] + "(" + makeString(param[1]) + ")"})  # noqa
     return string
 
 
@@ -813,11 +846,13 @@ class Authentication(object):
 
     _USERNAME = None
     _PASSWORD = None
+    _AUTH_DELEGATE = None
     _CERT = None
     _VERIFY = None
 
     def __init__(self, username=None, password=None,
-                 cert=None, verify=True, shared=False):
+                 cert=None, verify=True, shared=False,
+                 auth_delegate=None):
         '''
         :param str username=None: Username for basic authentication, None for
             unauthenticated access (or if using cert/verify)
@@ -832,12 +867,16 @@ class Authentication(object):
         :param bool shared=False: Set to True to make the values be class-level
             attributes (shared among instances where shared=True) instead of
             instance-level (shared=False, default)
+        :param AuthBase auth_delegate=None: Instance of requests' AuthBase to
+            allow arbitrary authentication schemes - mutually exclusive with
+            username/password arguments.
         '''
         self.shared = shared
         self.username = username
         self.password = password
         self.cert = cert
         self.verify = verify
+        self.auth_delegate = auth_delegate
 
     @property
     def username(self):
@@ -847,10 +886,15 @@ class Authentication(object):
 
     @username.setter
     def username(self, value):
-        if value is None:
-            pass
-        elif not isinstance(value, str):
-            raise TypeError('Value for "username" must be a str')
+        if value is not None:
+
+            if not isinstance(value, str):
+                raise TypeError('Value for "username" must be a str')
+
+            if self.auth_delegate is not None:
+                raise ValueError('Authentication instances may have username/password or auth_delegate set,'
+                                 ' but not both')
+
         if self.shared:
             self.__class__._USERNAME = value
         else:
@@ -864,10 +908,15 @@ class Authentication(object):
 
     @password.setter
     def password(self, value):
-        if value is None:
-            pass
-        elif not isinstance(value, str):
-            raise TypeError('Value for "password" must be a str')
+        if value is not None:
+
+            if not isinstance(value, str):
+                raise TypeError('Value for "password" must be a str')
+
+            if self.auth_delegate is not None:
+                raise ValueError('Authentication instances may have username/password or auth_delegate set,'
+                                 ' but not both')
+
         if self.shared:
             self.__class__._PASSWORD = value
         else:
@@ -927,8 +976,32 @@ class Authentication(object):
         else:
             self._verify = value
 
+    @property
+    def auth_delegate(self):
+        if self.shared:
+            return self._AUTH_DELEGATE
+        return self._auth_delegate
+
+    @auth_delegate.setter
+    def auth_delegate(self, value):
+        if value is not None:
+            if not isinstance(value, AuthBase):
+                raise TypeError('Value for "auth_delegate" must be an instance of AuthBase')
+
+            if self.username is not None or self.password is not None:
+                raise ValueError('Authentication instances may have username/password or auth_delegate set,'
+                                 ' but not both')
+
+        if self.shared:
+            self.__class__._AUTH_DELEGATE = value
+        else:
+            self._auth_delegate = value
+
     @property
     def urlopen_kwargs(self):
+        if self.auth_delegate is not None:
+            raise NotImplementedError("The urlopen_kwargs property is not supported when auth_delegate is set")
+
         return {
             'username': self.username,
             'password': self.password,
@@ -937,5 +1010,6 @@ class Authentication(object):
         }
 
     def __repr__(self, *args, **kwargs):
-        return '<{} shared={} username={} password={} cert={} verify={}>'.format(
-            self.__class__.__name__, self.shared, self.username, self.password, self.cert, self.verify)
+        return '<{} shared={} username={} password={} cert={} verify={} auth_delegate={}>'.format(
+            self.__class__.__name__, self.shared, self.username, self.password, self.cert, self.verify,
+            self.auth_delegate)
