@@ -23,11 +23,14 @@ def test_ogcapi_features_pygeoapi():
     assert paths['/collections/lakes'] is not None
 
     conformance = w.conformance()
-    assert len(conformance['conformsTo']) == 4
+    assert len(conformance['conformsTo']) == 9
 
     collections = w.collections()
     assert len(collections) > 0
 
+    feature_collections = w.feature_collections()
+    assert len(feature_collections) > 0
+
     lakes = w.collection('lakes')
     assert lakes['id'] == 'lakes'
     assert lakes['title'] == 'Large Lakes'
@@ -40,7 +43,7 @@ def test_ogcapi_features_pygeoapi():
     with pytest.raises(RuntimeError):
         lakes_query = w.collection_items('lakes', limit=0)
 
-    lakes_query = w.collection_items('lakes', limit=1)
+    lakes_query = w.collection_items('lakes', limit=1, admin='admin-0')
     assert lakes_query['numberMatched'] == 25
     assert lakes_query['numberReturned'] == 1
     assert len(lakes_query['features']) == 1
